c This program gives a correlation coefficient for data with errors. 
c
c Provie a data file named "dat.corr". The data format will be easily
c understood if you see a sample data attatched.
c Its main part is like this:
c
c  object_name(i) value_of_x(i) value_of_y(i)  value_of_ya(i) value_ofyb(i)
c
c where i=1,...n_data, the error indicated above means
c  value_of_ya(i) <  value_of_y(i) < value_of_yb(i) 

c In the first step of this program, the correlation coefficient 
c  "r" is obtained from  ( value_x(i), value_y(i) ).
c 
c In the next step, a set of fake data is created according
c to indicated error so that a number of "r" are created, and
c obtain a possible rage of "r" according to the significance
c level you need.. 
c
c Thing you must do.
c - provide data (x_i y_i) with error for y_i in "dat.corr"
c - give a sigma_value corresponding to your cofidence level when
c   you give the error in "dat.corr". eg.if 90%, 1.64 sigma.
c   this value must be set in subroutine crdata(). The default
c   value is 1.64, you can change it.
c   e.g.  double precision, parameter ::s90=1.64 
c What you obtain
c - in the file "r.result", you will see title r r1 and r2,
c where r is the correlation coefficiend and r1 < r <r2 in
c your confidence level.
c You may wish
c - see the scatter plot for "dat.corr" with simulated data
c      "plot test"
c - see how the rage of r  is determined, i.e., to see
c   cumulative distribution "plot rbunpu"
c
c module for random number creator
      module random         
        implicit none
        integer, parameter :: idummy=-223  !seed
        integer idum
      end module
c
c module for observed data
      module datafield
      integer n_data               !No. of objects
      real, dimension(100) :: x,y,a,b   !data x(i),y(i), err a<y<b
      character title*24           !comment e.g. xx.vs.yy
      real xm,sx,ym,sy,r           !mean value and standard deviation for x and y
      end module
c
c module for artificially created data
      module artfield
      real, dimension(100) :: u,v  !data x(i),y(i)
      real um,su,vm,sv,rr          !mean value and standard deviation for x and y
      end module                   !if needed, created data can be recoded 
                                   !in fort.1 in crdata
c
c module for cumulative distibution of simulated r
      module distribution
      parameter (ntry=10000 )
      real, dimension(ntry) :: rrr
      end module
c
c == Main Program ====================================================
      program statistical work
      use distribution
      use artfield, only : rr
      implicit none
      integer L
      open(1,file="r.result",status="replace")
      open(11,file="r.bunpu1",status="replace")
      open(12,file="r.bunpu2",status="replace")
      call initrand()   ! initialize the random number creator
      call rdata()      ! read data
      call anadata()    ! obtain corr. coef. r
      do L=1,ntry
        call crdata()     ! create a fake data set on "module datafield"
        call anadata2()   ! obtain corr. coef. r for artifical data
                          ! r is in fort.2
        rrr(L)=rr
      enddo
      call rtest()
  600 format(a,i4,f11.4)
      close(1)
      close(11)
      close(12)
      contains
c ======================================================
      subroutine rdata()
      use datafield
      implicit none
      character dummy*24
      integer i
      open (10,file="dat.corr",status="old")
      title=""
      read(10,*)dummy,title
      do i=1,100
        read(10,*,END=9)dummy,x(i),y(i),a(i),b(i)
c       write(6,*)dummy,x(i),y(i)
c       write(1,*)x(i),y(i)
      enddo
      stop
    9 continue
      close(10)
      n_data=i-1
      end
      subroutine anadata()
      use datafield
      implicit none
      call stat(x,n_data,xm,sx) !get mean value and s.d for x(n_data)
      call stat(y,n_data,ym,sy) !get mean value and s.d for y(n_data)
      call stat2(x,y,n_data,xm,sx,ym,sy,r) !get correlation r
      write(1,600)title,n_data,r
  600 format(a,i4,f13.5)
c 600 format(a,"n=",i4,"  r=",f11.4)
c     write(6,601)n_data,r,xm,sx,ym,sy
c 600 format("n_data=",i5,"corr.=",f10.5/
c    & "<x>=",f10.5," Sx=",f10.5/"<y>=",f10.5," Sy=",f10.5)
      end
      subroutine anadata2()
      use datafield, only:n_data
      use artfield
      implicit none
      call stat(u,n_data,um,su) !get mean value and s.d for x(n_data)
      call stat(v,n_data,vm,sv) !get mean value and s.d for y(n_data)
      call stat2(u,v,n_data,um,su,vm,sv,rr) !get correlation r
c     write(6,600)n_data,rr,um,su,vm,sv
c     write(6,601)n_data,rr
c 600 format("n_data=",i5,"corr.=",f10.5/
c    & "<u>=",f10.5," Su=",f10.5/"<v>=",f10.5," Sv=",f10.5)
c 601 format(i4,f10.5)
      end
c ---------------------------------------------------
c get mean value and standard deviation for array x()
      subroutine stat(x,n,xm,xs)
      implicit none
      integer n
      real, dimension(n) :: x
      real xm,xs
      real sum1,sum2
      integer i
      sum1=0.0
      sum2=0.0
      do i=1,n
        sum1=sum1+x(i)
        sum2=sum2+x(i)*x(i)
      enddo
      xm=sum1/real(n)
      xs = sqrt((sum2 - xm*xm*real(n))/real(n-1))
      end
c ---------------------------------------------------
c get cerrelation coefficient r 
      subroutine stat2(x,y,n,xm,sx,ym,sy,r) !get correlation r
      implicit none
      integer n
      real, dimension(n) :: x,y
      real xm,sx,ym,sy,r
      real sum1
      integer i
      sum1=0.0
      do i=1,n
        sum1=sum1+x(i)*y(i)
      enddo
      r = (sum1 - real(n)*xm*ym )/real(n-1)/sx/sy
      end
c ---------------------------------------------------
      subroutine crdata()
      use random
      use datafield
      use artfield
      implicit none
      double precision, parameter ::ss=1.64 
      integer i
      double precision delta
      do i=1,n_data
        u(i)=x(i)
        delta=seikib()
c       delta= -1.64d0 
        if(delta.lt.0.d0) then
           delta=delta*(y(i)-a(i))/ss
        else
           delta=delta*(b(i)-y(i))/ss
        endif
        v(i)=y(i)+delta
c       write(9,*)u(i),v(i)
      enddo
      end
c ----------------------------------------------------
      subroutine rtest()
      use distribution
      implicit none
      integer i,ic,jc
      real f,r1,r2
      call sort(ntry,rrr)
      ic=0
      jc=0
      do i=1,ntry
        f=real(i)/real(ntry)
        write(11,600)rrr(i),f
        if(ic.eq.0 .and. f.gt.0.05)  then
          ic =1
          r1=0.5*(rrr(i)+rrr(i+1))
        endif
        if(jc.eq.0 .and. f.gt.0.95)  then
          jc=1
          r2=0.5*(rrr(i)+rrr(i-1))
        endif
      enddo
      write(12,602)r1,r1
      write(12,602)r2,r2
      write(1,601)r1,r2
c 601 format("($ ",f15.5," < r < ",f15.5," $)")
  601 format("  the_rage_of_r",2f13.5)
  600 format(2f15.5)
  602 format(/f15.5, " 0.0"/ f15.5, " 1.0")
      end
c ----------------------------------------------------
c Initialize the random number creater.
c note: never change idum after the first initilization.
      subroutine initrand()
c ----------------------------------------------------
      use random
      implicit none
      real r
      idum=idummy
      r=ran2(idum)
      end
c ----------------------------------------------------
c A simple normal distribution: 
c mean value = 1.0, standard deviation = 1.0
      real function seikib()
c ----------------------------------------------------
      use random
      implicit none
      integer j
      seikib=0.0
      do j=1,12
        seikib=seikib+ran2(idum)
      enddo
      seikib=seikib-6.0
      end
c---------------------------------------------------------------------
      FUNCTION ran2(idum)
c random value creator: 0< ran2 <1 uniform distribution
c---------------------------------------------------------------------
      INTEGER idum,IM1,IM2,IMM1,IA1,IA2,IQ1,IQ2,IR1,IR2,NTAB,NDIV
      REAL ran2,AM,EPS,RNMX
      PARAMETER (IM1=2147483563,IM2=2147483399,AM=1./IM1,IMM1=IM1-1,
     &     IA1=40014,IA2=40692,IQ1=53668,IQ2=52774,IR1=12211,
     &     IR2=3791,NTAB=32,NDIV=1+IMM1/NTAB,EPS=1.2e-7,RNMX=1.-EPS)
      INTEGER idum2,j,k,iv(NTAB),iy
      SAVE iv,iy,idum2
      DATA idum2/123456789/, iv/NTAB*0/, iy/0/
      if (idum.le.0) then
          idum=max(-idum,1)
          idum2=idum
          do j=NTAB+8,1,-1
              k=idum/IQ1
              idum=IA1*(idum-k*IQ1)-k*IR1
              if (idum.lt.0) idum=idum+IM1
              if (j.le.NTAB) iv(j)=idum
          enddo
          iy=iv(1)
      endif
      k=idum/IQ1
      idum=IA1*(idum-k*IQ1)-k*IR1
      if (idum.lt.0) idum=idum+IM1
      k=idum2/IQ2
      idum2=IA2*(idum2-k*IQ2)-k*IR2
      if (idum2.lt.0) idum2=idum2+IM2
      j=1+iy/NDIV
      iy=iv(j)-idum2
      iv(j)=idum
      if(iy.lt.1)iy=iy+IMM1
      ran2=min(AM*iy,RNMX)
      return
      END
      SUBROUTINE sort(n,arr)
      INTEGER n,M,NSTACK
      REAL arr(n)
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,ir,j,jstack,k,l,istack(NSTACK)
      REAL a,temp
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 12 j=l+1,ir
          a=arr(j)
          do 11 i=j-1,l,-1
            if(arr(i).le.a)goto 2
            arr(i+1)=arr(i)
11        continue
          i=l-1
2         arr(i+1)=a
12      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        temp=arr(k)
        arr(k)=arr(l+1)
        arr(l+1)=temp
        if(arr(l).gt.arr(ir))then
          temp=arr(l)
          arr(l)=arr(ir)
          arr(ir)=temp
        endif
        if(arr(l+1).gt.arr(ir))then
          temp=arr(l+1)
          arr(l+1)=arr(ir)
          arr(ir)=temp
        endif
        if(arr(l).gt.arr(l+1))then
          temp=arr(l)
          arr(l)=arr(l+1)
          arr(l+1)=temp
        endif
        i=l+1
        j=ir
        a=arr(l+1)
3       continue
          i=i+1
        if(arr(i).lt.a)goto 3
4       continue
          j=j-1
        if(arr(j).gt.a)goto 4
        if(j.lt.i)goto 5
        temp=arr(i)
        arr(i)=arr(j)
        arr(j)=temp
        goto 3
5       arr(l+1)=arr(j)
        arr(j)=a
        jstack=jstack+2
        if(jstack.gt.NSTACK)write(6,*) 'NSTACK too small in sort'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END
      end program
